# Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved. 
# Caution: This file is part of the command scripting implementation. Do not edit or move
# this file because this may cause commands and scripts to fail. Do not
# try to reuse the logic in this file or keep copies of this file because this
# could cause your scripts to fail when you upgrade to a different version.
"""
This script holds all the commands for Webcenter WorkList Connections.
@author Alexis Chan
"""
import ora_mbs
import javax.management as management
import jarray
from java.lang import Object
from java.lang import String
from java.lang import Exception
from javax.management import Attribute
import WebCenterHandlerHelper as helper
# Code for detecting syntax error, especially missing colons
def bpelCheck():
    helper.wcPrintMessage('FILE_CHECK_MESG')

# Create a BPEL Connection.
def createBPELConnection(appName,name,url,policy='',recipientKeyAlias='',linkUrl='',server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    noError = 1
    nameError = 0
    urlError = 0
    if (not checkBPELFieldNotEmpty(name)):
        nameError=1
        noError=0
    if (not checkBPELFieldNotEmpty(url)):
        urlError=1
        noError=0
    if noError==0:
        helper.wcPrintErrorMessage('CREATECONNECTION_ARGUMENT_ERROR')
        if nameError:
            printInvalidBPELConnectionName()
        if urlError:
            printBPELInvalidUrl()
    if noError:
        connectionType='BPEL'
        # Default the policy if it has not been set
        if policy == '':
            policy='oracle/wss10_saml_token_client_policy'
        userArg={'URL':url,'PolicyURI':policy,'RecipientKeyAlias':recipientKeyAlias,'LinkURL':linkUrl} # userArg is a dictionary
        helper.customCreateConnection(appName,name,connectionType,userArg)
    helper.webcenterRestore()

# Set a BPEL Connection.
def setBPELConnection(appName,name,url=None,policy=None,recipientKeyAlias=None,linkUrl=None,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    noError = 1
    nameError = 0
    urlError = 0
    if (not checkBPELFieldNotEmpty(name)):
        nameError=1
        noError=0
    if noError == 0:
        helper.wcPrintErrorMessage('SETCONNECTION_ARGUMENT_ERROR')
        if nameError:
            printInvalidBPELConnectionName()
        if urlError:
            printBPELInvalidUrl()
    if noError:
        connectionType='BPEL'
        userArg={}
        if (url != None):
            userArg.update({'URL':url})
        if (policy != None):
            userArg.update({'PolicyURI':policy})
        if (recipientKeyAlias != None):
            userArg.update({'RecipientKeyAlias':recipientKeyAlias})
        if (linkUrl != None):
            userArg.update({'LinkURL':linkUrl})
        helper.customSetConnection(appName,name,connectionType,userArg,0)
    helper.webcenterRestore()

# List all the connections of type BPEL
def listBPELConnections(appName,verbose=0,name='',server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    # stringAttrs is a dictionary.
    stringAttrs=[(helper.wcGetMessageFromResourceBundle('BPEL_CONNECTION_NAME_ID1'),'ConnectionName'), \
                 (helper.wcGetMessageFromResourceBundle('BPEL_POLICY_URI_ID2'),'PolicyURI'), \
                 (helper.wcGetMessageFromResourceBundle('BPEL_URL_ID3'),'URL'),  \
                 (helper.wcGetMessageFromResourceBundle('BPEL_RECIPIENT_KEY_ALIAS_ID4'),'RecipientKeyAlias'),  \
                 (helper.wcGetMessageFromResourceBundle('BPEL_LINK_URL_ID5'),'LinkURL')]

    # nonStrAttrs is supposed to be a dictionary too but since there are no non-String attributes to be listed, it is set to the null object.
    nonStrAttrs=None
    helper.customListConnections(appName,'BPEL',stringAttrs,bpelNonStrAttrs,nonStrAttrs,verbose,name)
    helper.webcenterRestore()

# Adds a BPEL connection into the ADF Worklist Config MBean.
def addWorklistConnection(appName,name,verbose=0,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    createWorklistEntry(appName,name,'addConnectionName',printAddToWorkListSuccess,helper.printErrorInvokingAddConnectionName,verbose)
    if verbose:
        listBPELConnections(appName,1,name,server,applicationVersion)
    helper.webcenterRestore()

# Removes the specified connection from the ADF Worklist Config MBean
def removeWorklistConnection(appName,name,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    removeWorklistEntry(appName,name,'removeConnectionName',printRemoveFromWorkListSuccess,helper.printErrorInvokingRemoveConnectionName)
    helper.webcenterRestore()

# Lists BPEL Connections in the ADF Worklist Config MBean
def listWorklistConnections(appName,verbose=0,name='',server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    noError = 1
    connectionType='BPEL'
    # Get Worklist Config MBean's Object Name
    WorklistConfigObjName = getWorklistConfigObjName(appName)
    # Get Connection Names listed in Worklist Config MBean
    # as JDev wizard entries can have spaces and odd chars in them strip the 
    # values and create a new array.
    try:
        worklistConnectionsUnclean = ora_mbs.getAttribute(WorklistConfigObjName,'ConnectionNames')
        worklistConnections = []
        for n in worklistConnectionsUnclean:
            worklistConnections.append(n.strip())
    except Exception,ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_INVOKING_GETCONNECTIONNAMES')
        noError=0
    if noError:
        if name!='':
            connectionsNotInWorklist = []
            if name in worklistConnections:
                listBPELConnections(appName,verbose,name,server,applicationVersion)
            else:
                printMissingWorkListConnection(name)
        else: # If no user-specified connections, just print the connections in the Worklist
            if len(worklistConnections)>0:
                for n in worklistConnections:
                        listBPELConnections(appName,verbose,n,server,applicationVersion)
            else:
                printNoConnectionsInWorklist()
    helper.webcenterRestore()
    
# List BPEL Connection in the ADF Task Config MBean
def listTaskConnection(appName,verbose=0,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    connectionType='BPEL'
    # Get Worklist Config MBean's Object Name
    TaskConfigObjName = getTaskConfigObjName(appName)
    # Get Connection Name listed in Task Config MBean
    try:
        taskConnection = ora_mbs.getAttribute(TaskConfigObjName,'ConnectionName')
        taskConnection = taskConnection.strip()
        if taskConnection=='':
            printNoTaskConnection()
        else:
            listBPELConnections(appName,verbose,taskConnection,server,applicationVersion)
    except Exception,ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        printErrorGettingConnectionName()
    helper.webcenterRestore()

# Sets the specified BPEL Connection as the Task Connection in ADF Task Config MBean
def setTaskConnection(appName,name,verbose=0,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    noError = 1
    connectionType='BPEL'
    connectionMBeans = helper.getMBeanArrayInADFConnections()
    ADFConnObjName = helper.getADFConnectionObjName(connectionMBeans,appName)
    # Need the ADF Connections Object Name to search for the connection name
    if ADFConnObjName is None:
        noError=0
    if noError:
        # If connection does not exist, create it. 
        if not helper.connMBeanExists(ADFConnObjName,name,connectionType) and name !='' :
            createBPELConnection(appName,name,'')
        # Set 'ConnectionName' in ADFTaskConfig to name
        # If the connection name is an empty string, set 'ConnectionName' in ADFTaskConfig to '' 
        TaskConfigObjName = getTaskConfigObjName(appName)
        try:
            ora_mbs.setAttribute(TaskConfigObjName,Attribute('ConnectionName',name))
            helper.customSaveConfig(appName)
            if name == '':
                printTaskConnectionDisabled()
            else:
                printSetAsTaskConnection(name)
            if verbose and name != '':
                listBPELConnections(appName,verbose,name,server,applicationVersion)
        except Exception,ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            printErrorSettingTaskConnection()
            helper.customRefreshConfig(appName)
    helper.webcenterRestore()

# Helper functions

# Returns the Task Config MBean's Object Name if it exists
def getTaskConfigObjName(appName):
    return helper.findOrCreateConfigObjName(appName,'oracle.adf.mbean.share.config.runtime.webcenter.collab.task.AdfTaskConfig' )

# Returns the Worklist Config MBean's Object Name if it exists
def getWorklistConfigObjName(appName):
    return helper.findOrCreateConfigObjName(appName,'oracle.adf.mbean.share.config.runtime.webcenter.worklist.AdfWorklistConfig' )

def getBPELDefaultConnection(appName):
    bpelConfigObj = getWorklistConfigObjName(appName)
    if bpelConfigObj is not None:
        try:
            connNames = ora_mbs.getAttribute(bpelConfigObj,'ConnectionNames')
            return connNames
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
	    helper.wcPrintErrorMessage('ERROR_GET_ATTRIBUTE','ConfigChildMBeans')
    else:
        helper.wcPrintMessage('NONE')
    return None

def getWorklistAttribute(appName, attr_name):
    worklistConfigObj = getWorklistConfigObjName(appName)
    if worklistConfigObj is not None:
        try:
            attr_value = ora_mbs.getAttribute(worklistConfigObj,attr_name)
            return attr_value
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
	    helper.wcPrintErrorMessage('ERROR_GET_ATTRIBUTE','ConfigChildMBeans')
    else:
        helper.wcPrintMessage('NONE')
    return None

def getTaskAttribute(appName, attr_name):
    taskConfigObj = getTaskConfigObjName(appName)
    if taskConfigObj is not None:
        try:
            attr_value = ora_mbs.getAttribute(taskConfigObj,attr_name)
            return attr_value
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
	    helper.wcPrintErrorMessage('ERROR_GET_ATTRIBUTE','ConfigChildMBeans')
    else:
        helper.wcPrintMessage('NONE')
    return None


# Helper function for addWorklistConnection()
def createWorklistEntry(appName,connectionName,invokeStr,successFunc,errorFunc,verbose=0):
    connectionType='BPEL'
    connectionMBeans = helper.getMBeanArrayInADFConnections()
    ADFConnObjName = helper.getADFConnectionObjName(connectionMBeans,appName)
    # Need the ADF Connections Object Name to search for the connection name
    if ADFConnObjName is None:
        return
    # If the connection exists, invoke addConnectionName in ADFWorklistConfig
    if helper.connMBeanExists(ADFConnObjName,connectionName,connectionType):
        WorklistConfigObjName = getWorklistConfigObjName(appName)
        params= jarray.array([connectionName],Object)
        paramTypes= jarray.array(['java.lang.String'],String)
        # Invoke ADFWorklistConfig's 'addConnectionName'
        try:
            ora_mbs.invoke(WorklistConfigObjName,invokeStr,params,paramTypes)
            helper.customSaveConfig(appName)
            # To check whether the addition or removal is successful ,  ora_mbs.getAttribute(WorklistConfigObjName,'ConnectionNames')
            successFunc(connectionName)
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            errorFunc()
    else:
        helper.wcPrintErrorMessage('ERROR_NO_CONNECTION',connectionName)


# Helper function for removeWorklistConnection()
def removeWorklistEntry(appName,connectionName,invokeStr,successFunc,errorFunc,verbose=0):
    noError=1
    WorklistConfigObjName = getWorklistConfigObjName(appName)
    # Get Connection Names listed in Worklist Config MBean
    try:
        worklistConnections = ora_mbs.getAttribute(WorklistConfigObjName,'ConnectionNames')
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_INVOKING_GETCONNECTIONNAMES')
        noError=0
    if noError:
        if len(connectionName)>0:
            connectionsNotInWorklist = []
            # Specify connections that are not in the worklist
            found = 'no'
            for n in  worklistConnections:
                ns = n.strip() 
                if ns == connectionName:
                    found = 'yes'
                    try:
                        # We need to use the untrimmed string to make sure the correct named
                        # entry is removed.
                        params= jarray.array([n],Object)
                        paramTypes= jarray.array(['java.lang.String'],String)
                        ora_mbs.invoke(WorklistConfigObjName,invokeStr,params,paramTypes)
                        helper.customSaveConfig(appName)
                        # To check whether the addition or removal is successful ,  ora_mbs.getAttribute(WorklistConfigObjName,'ConnectionNames')
                        successFunc(connectionName)
                    except Exception, ex:
                        helper.wcPrintExceptionMessage(ex.getMessage())
                        errorFunc()
            if found == 'no':
                helper.wcPrintMessage('BPEL_WORKLIST_ENTRY_MISSING',connectionName)
        else: # If no user-specified connections, just print the connections in the Worklist
            printNoConnectionsInWorklist()    
    else:
        helper.wcPrintErrorMessage('ERROR_NO_CONNECTION',connectionName)

#Function to check if param is not null or not empty
def checkBPELFieldNotEmpty(param):
    if param is None:
        return 0
    if len(param.strip())==0:
        return 0
    return 1

# Function to be passed by listBPELConnections to helper.customListConnections.
# helper.customListConnections requires a function to be passed to it. It calls that function to list the non-String attributes of a connection. Worklist and Tasks does not list non-String attributes, so issue the 'pass' statement.
def bpelNonStrAttrs(objName,nonStrAttrs):
    pass # pass is a valid Python statement that does nothing.

#Messages for Worklist Service
def printTaskConnectionDisabled():
    helper.wcPrintMessage('BPEL_TASK_SERVICE_DISABLED')

def printNoTaskConnection():
    helper.wcPrintMessage('BPEL_TASK_CONNECTION_MISSING')

def printSetAsTaskConnection(connectionName):
    helper.wcPrintMessage('BPEL_TASK_CONNECTION_SUCCESS',connectionName)
    helper.wcPrintMessage('INFORM_NEED_RESTART')

def printErrorSettingTaskConnection():
    helper.wcPrintErrorMessage('BPEL_TASK_CONNECTION_ERROR')

def printRemoveFromWorkListSuccess(connectionName):
    helper.wcPrintMessage('BPEL_WORKLIST_CONNECTION_REMOVED',connectionName)
    helper.wcPrintMessage('INFORM_NEED_RESTART')

def printAddToWorkListSuccess(connectionName):
    helper.wcPrintMessage('BPEL_WORKLIST_CONNECTION_ADDED',connectionName)
    helper.wcPrintMessage('INFORM_NEED_RESTART')

def printErrorCdToWorkListConfig():
    helper.wcPrintErrorMessage('BPEL_WORKLIST_CONIG_ERROR')

def printErrorCdToTaskConfig():
    helper.wcPrintErrorMessage('BPEL_TASK_CONFIG_ERROR')

def printNoConnectionsInWorklist():
    helper.wcPrintMessage('BPEL_WORKLIST_NO_CONNECTIONS')

def printNoValidConnectionsProvided():
    helper.wcPrintMessage('BPEL_NO_VALID_CONNECTIONS')

def printErrorGettingConnectionName():
    helper.wcPrintErrorMessage('BPEL_ERROR_GETTING_CONNECTION_NAME')

def printInvalidBPELConnectionName():
    helper.wcPrintErrorMessage('BPEL_ERROR_INVALID_CONNECTION_NAME')

def printBPELInvalidUrl():
    helper.wcPrintErrorMessage('BPEL_ERROR_INVALID_URL')

def printMissingWorkListConnection(connectionName):
    helper.wcPrintMessage('BPEL_WORKLIST_ENTRY_MISSING',connectionName)

def getBPELObjectName(appName, connectionName):
    mbeanarray = helper.getMBeanArrayInADFConnections()
    ADFConnObjName = helper.getADFConnectionObjName(mbeanarray,appName)
    connObjName = helper.getConnObjName(ADFConnObjName,connectionName,'BPEL')
    return connObjName
    
	
def getBPELConnectionAttribute(appName, connectionName, attributeType, server='', applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    attrVal = None
    try:
        objName = getBPELObjectName(appName, connectionName)
        if objName is None :
            attrVal = None
        attrVal = ora_mbs.getAttribute(objName, attributeType)
    except Exception,ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_GET_ATTRIBUTE',attributeType)
    helper.webcenterRestore()
    return attrVal

def getWorklistConnections(appName,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    connectionType='BPEL'
    # Get Worklist Config MBean's Object Name
    WorklistConfigObjName = getWorklistConfigObjName(appName)
    # Get Connection Names listed in Worklist Config MBean
    # as JDev wizard entries can have spaces and odd chars in them strip the 
    # values and create a new array.
    try:
        worklistConnectionsUnclean = ora_mbs.getAttribute(WorklistConfigObjName,'ConnectionNames')
        worklistConnections = []
        for n in worklistConnectionsUnclean:
            worklistConnections.append(n.strip())
    except Exception,ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_INVOKING_GETCONNECTIONNAMES')
    helper.webcenterRestore()
    return worklistConnections
