# Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved. 
# Caution: This file is part of the command scripting implementation. Do not edit or move
# this file because this may cause commands and scripts to fail. Do not
# try to reuse the logic in this file or keep copies of this file because this
# could cause your scripts to fail when you upgrade to a different version. 
import ora_mbs
import WebCenterHandlerHelper as helper
import ProducerHelperHandler as producerhelper
from javax.management import RuntimeMBeanException
from javax.management.openmbean import OpenType
from javax.management.openmbean import ArrayType
from javax.management.openmbean import CompositeType
from javax.management.openmbean import SimpleType
from javax.management.openmbean import CompositeDataSupport
from javax.management.openmbean import CompositeData

from java.lang.reflect import Array
from java.util import UUID
from java.lang import String
from java.lang import Object
from java.lang import Exception
import jarray
import re

# Code for detecting syntax error, especially missing colons
def wsrpCheck():
    helper.wcPrintMessage('FILE_CHECK_MESG')

def registerWSRPProducer(appName, name, url, proxyHost=None, proxyPort=-1, timeout=30, externalApp=None,
                         registrationProperties=None, tokenType=None, issuer=None, defUser=None,
                         keyStorePath=None, keyStorePswd=None, sigKeyAlias=None, sigKeyPswd=None, encKeyAlias=None,
                         encKeyPswd=None, recptAlias=None,server='',applicationVersion='', enforcePolicyURI=1):
    helper.webcenterInitialize(server,applicationVersion)
    # The producer name is the same as the connection name
    wsrpConnName = name
    wsConnName = name + '-wsconn'

    regProps = None
    if registrationProperties is not None:
        # Convert the registrationProperties string (e.g. prop1=value1, 
        # prop2=value2) into a list, ['prop1=value1', 'prop2=value2']
        propList = registrationProperties.split('\,')

        # Convert the properties list into a collection, {'prop1':'value1' 
        # 'prop2':'value2'}
        propDict = {}
        for prop in propList:
            splitProp = prop.split('=')
            propDict[splitProp[0]] = splitProp[1]

        # Convert the property dictionary into a list of CompositeData objects
        regProps = _createRegPropertyCompositeDataList(propDict)

    if _createWSRPConnection(appName, url, name, wsrpConnName, wsConnName, externalApp, proxyHost, proxyPort,
                             timeout, regProps, tokenType, issuer, defUser, keyStorePath, keyStorePswd,
                             sigKeyAlias, sigKeyPswd, encKeyAlias, encKeyPswd, recptAlias,server,applicationVersion, enforcePolicyURI):
        if not producerhelper.registerProducer(appName, wsrpConnName,server,applicationVersion):
            helper.deleteConnection(appName, wsrpConnName,server,applicationVersion)
            helper.deleteConnection(appName, wsConnName,server,applicationVersion)

    helper.webcenterRestore()

def _createWSRPConnection(appName, wsdl, producerName, wsrpConnName, wsConnName, extAppId=None, proxyHost=None,
                          proxyPort=0, timeout=30, regProps=None, tokenType=None, issuer=None, defUser=None,
                          keyStorePath=None, keyStorePswd=None, sigKeyAlias=None, sigKeyPswd=None,
                          encKeyAlias=None, encKeyPswd=None, recptAlias=None,server='',applicationVersion='', enforcePolicyURI=1):
    helper.webcenterInitialize(server,applicationVersion)
    model = _getWsdlModel(appName, wsdl, proxyHost, proxyPort)
    if model is None:
        helper.webcenterRestore()
        return 0

    # Check if the tokenType is valid in case of secure producers
    policyName = tokenType
    if enforcePolicyURI and tokenType is not None and tokenType != '':
        policyName = producerhelper.getPolicyName(tokenType)
        if policyName is None:
            return 0 

    connectionMBeans = helper.getMBeanArrayInADFConnections()
    adfConnBean = helper.getADFConnectionObjName(connectionMBeans,appName)
    # Need the ADF Connections Object Name to search for the connection name
    if adfConnBean is None:
        return 0
    
    # Check for an existing WS connection with that name
    if (producerhelper.producerConnectionExists(adfConnBean, 'WebServiceConnection', wsConnName)):
        helper.wcPrintMessage('WS_CONN_ALREADY_EXIST', wsConnName + ',' + wsrpConnName)
        helper.webcenterRestore()
        return 0

    # Check for an existing WSRP connection with that name
    if (producerhelper.producerConnectionExists(adfConnBean, 'WSRPProducerConnection', wsrpConnName)):
        helper.wcPrintMessage('WSRP_CONN_ALREADY_EXIST', wsrpConnName)
        helper.webcenterRestore()
        return 0

    # Create a web service connection using the WSDL model
    wsArgs = {'WsdlUrl':wsdl, 'Model':model}
    wsConnObj = helper.customCreateConnection(appName, wsConnName, 'WebServiceConnection', wsArgs,needsRestart=0)
    if wsConnObj is None:
        return 0

    _setNoAuthOnNonMarkupWebServicePortMBeans(wsConnObj)
     
    # If ws-security params are passed 
    if tokenType is not None and tokenType != '':
        # Fetch the Port mbean from wsConnObj
        portMbeanObj = producerhelper.getWebServicePortMBean(wsConnObj)

        if portMbeanObj is not None:
            # Set the ws-sec attributes on the Port Mbean
            if _setWsSecProperties(portMbeanObj, policyName, issuer, defUser, keyStorePath,
                                   keyStorePswd, sigKeyAlias, sigKeyPswd, encKeyAlias, encKeyPswd, recptAlias):
                helper.wcPrintMessage('UPDATED_WSSEC_PROP_ON_CONN_PORT', wsConnName)
            else:
                helper.wcPrintErrorMessage('ERROR_UPDATING_WSSEC_PROP_ON_CONN_PORT', wsConnName)
                return 0
        else:
           helper.wcPrintMessage('PORT_MBEAN_NOT_EXIST', wsConnName)
           return 0

    # Create a WSRP connection that uses the web service connection
    regPropArray = None
    if regProps is not None:
        regPropArray = jarray.array(regProps, CompositeData)

    wsrpArgs = producerhelper.createArgMap({'WSConnectionName':wsConnName, 'ProducerName':producerName, 'Timeout':timeout, 'ProxyHost':proxyHost, 'ProxyPort':proxyPort, 'ExtAppId':extAppId, 'RegistrationProperties':regPropArray, 'WSDefaultUser':defUser})

    helper.customCreateConnection(appName, wsrpConnName, 'WSRPProducerConnection', wsrpArgs,needsRestart=0)

    helper.webcenterRestore()
    return 1

#Removed dumpStack because it does not print the stack trace from MBean anyway. To print stack trace from MBean use helper.wcPrintExceptionMessage(ex.printStackTrace())

# Add ws-sec properties on Port Mbean
def _setWsSecProperties(portMbeanObj, tokenType=None, issuer=None, defUser=None,
                        keyStorePath=None, keyStorePswd=None, sigKeyAlias=None, sigKeyPswd=None,
                        encKeyAlias=None, encKeyPswd=None, recptAlias=None):

    try:
        if tokenType is not None:
            # Add Policy reference
            objs = jarray.array([String('security'),String(tokenType)],Object)
            strs = jarray.array(['java.lang.String','java.lang.String'],String)
            ora_mbs.invoke(portMbeanObj, 'addPolicyReference',objs,strs)
    except Exception, ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_ADDING_POLICY_REF')
        return 0
    # Intialize the dictionary with ws-sec property/value pair
    portCallProps = {'oracle.webservices.security.saml.issuer.name':issuer,
                     'javax.xml.rpc.security.auth.username':defUser,
                     'oracle.webservices.security.keystore.type':'JKS',
                     'oracle.webservices.security.keystore.location':keyStorePath,
                     'oracle.webservices.security.keystore.password':keyStorePswd,
                     'oracle.webservices.security.encryption.key.alias':encKeyAlias,
                     'oracle.webservices.security.encryption.key.password':encKeyPswd,
                     'oracle.webservices.security.signature.key.alias':sigKeyAlias,
                     'oracle.webservices.security.signature.key.password':sigKeyPswd,
                     'oracle.webservices.security.recipient.key.alias':recptAlias}

    # Call addCallProperty on Port mbean to add ws-sec properties and value
    for x in portCallProps.items():
        try:
            if x[1] is not None:
                objs = jarray.array([String(x[0]),String(x[1])],Object)
                strs = jarray.array(['java.lang.String','java.lang.String'],String)
                ora_mbs.invoke(portMbeanObj, 'addCallProperty', objs, strs)
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            helper.wcPrintErrorMessage('ERROR_SETTING_CALL_PROP_VALUE', x[1])
            return 0
    return 1

def setWSRPProducer(appName, name, url=None, proxyHost=None, proxyPort=-1, timeout=-1, externalApp=None,server='',
                    applicationVersion='', tokenType=None, issuer=None, defUser=None, keyStorePath=None, keyStorePswd=None,
                    sigKeyAlias=None, sigKeyPswd=None, encKeyAlias=None, encKeyPswd=None, recptAlias=None, enforcePolicyURI=1): 

    helper.webcenterInitialize(server,applicationVersion)
    connectionMBeans = helper.getMBeanArrayInADFConnections()
    adfConnBean = helper.getADFConnectionObjName(connectionMBeans,appName)
    # Need the ADF Connections Object Name to search for the connection name
    if adfConnBean is None:
        helper.webcenterRestore()
        return
    
    if not producerhelper.producerConnectionExists(adfConnBean, 'WSRPProducerConnection', name):
        producerhelper.noProducerConnectionErrorMsg('WSRPProducerConnection', name)
        helper.webcenterRestore()
        return

    # Get the WSRP connection object
    wsrpObj = producerhelper.producerGetConnectionObject(appName, name, 'WSRPProducerConnection')
    
    if url is not None:
        model = _getWsdlModel(appName, url, proxyHost, proxyPort)
        if model is None:
            helper.webcenterRestore()
            return 0

        # Get the attached WS connection name 
        wsName = ora_mbs.getAttribute(wsrpObj, 'WSConnectionName')

        # Delete the WS connection
        helper.deleteConnection(appName, wsName,server,applicationVersion)
        # START WORKAROUND DELETECONNECTION RESETS SERVER AND APPLICATION VERSION 
        helper.WebCenter.server=server
        helper.WebCenter.applicationVersion=applicationVersion       
        # END WORKAROUND
        # Create the WS connection with the same name but a different WSDL URL
        wsArgs = {'WsdlUrl':url, 'Model':model}
        wsConnObj = helper.customCreateConnection(appName, wsName, 'WebServiceConnection', wsArgs,needsRestart=0)
        _setNoAuthOnNonMarkupWebServicePortMBeans(wsConnObj)
        helper.customSave(appName)


    # Set up any WSRP connection attributes that might have been set
    wsrpArgs = producerhelper.createArgMap({'ProxyHost':proxyHost, 'ProxyPort':proxyPort, 'ExtAppId':externalApp, 'Timeout':timeout, 'WSDefaultUser':defUser})

    if wsrpArgs is not None:
        helper.customSetConnection(appName, name, 'WSRPProducerConnection', wsrpArgs, 0,needsRestart=0)
    # Proceed only if any of the ws security properties need to be updated
    if tokenType is None and issuer is None and defUser is None and keyStorePath is None and \
       keyStorePswd is None and sigKeyAlias is None and sigKeyAlias is None and \
       sigKeyPswd is None and encKeyAlias is None and encKeyPswd is None:
         return;

    # Get the WebService connection object
    wsName = ora_mbs.getAttribute(wsrpObj, 'WSConnectionName')
    wsConnObj = producerhelper.producerGetConnectionObject(appName, wsName, 'WebServiceConnection')
    # Get the connection Port on which ws-sec properties need to be updated
    portMbeanObj = producerhelper.getWebServicePortMBean(wsConnObj);
    if portMbeanObj is  None:
        helper.wcPrintMessage('PORT_MBEAN_NOT_EXIST', wsName);
        return 0;

    # Set the ws-sec attributes on the Port Mbean
    if _updateWsSecProperties(portMbeanObj, tokenType, issuer, defUser, keyStorePath,
                           keyStorePswd, sigKeyAlias, sigKeyPswd, encKeyAlias, encKeyPswd, recptAlias, enforcePolicyURI):
        helper.customSave(appName)
        helper.wcPrintMessage('UPDATED_WSSEC_PROP_ON_CONN_PORT', wsName)
        return 1
    else:
        helper.wcPrintErrorMessage('ERROR_UPDATING_WSSEC_PROP_ON_CONN_PORT', wsName)
        return 0
    helper.webcenterRestore()

# Update ws-sec properties on Port Mbean
def _updateWsSecProperties(portMbeanObj, tokenType=None, issuer=None, defUser=None,
                        keyStorePath=None, keyStorePswd=None, sigKeyAlias=None, sigKeyPswd=None,
                        encKeyAlias=None, encKeyPswd=None, recptAlias=None, enforcePolicyURI=1):

   # Intialize the dictionary with ws-sec property/value pair
    portCallProps = {'oracle.webservices.security.saml.issuer.name':issuer,
                     'javax.xml.rpc.security.auth.username':defUser,
                     'oracle.webservices.security.keystore.type':'JKS',
                     'oracle.webservices.security.keystore.location':keyStorePath,
                     'oracle.webservices.security.keystore.password':keyStorePswd,
                     'oracle.webservices.security.encryption.key.alias':encKeyAlias,
                     'oracle.webservices.security.encryption.key.password':encKeyPswd,
                     'oracle.webservices.security.signature.key.alias':sigKeyAlias,
                     'oracle.webservices.security.signature.key.password':sigKeyPswd,
                     'oracle.webservices.security.recipient.key.alias':recptAlias}

    # If token type is passed check if it maps to a valid policy
    policyName = tokenType
    if enforcePolicyURI and tokenType is not None and tokenType != '':
        policyName = producerhelper.getPolicyName(tokenType)
        if policyName is None:
            return 0 

    # If token type is blank remove all the ws-sec properties.
    # Otherwise drop and recreate the policy reference
    if tokenType == '' or tokenType is not None:
        try:
            policyStr = ora_mbs.getAttribute(portMbeanObj, 'PolicyReferencesAsStringArray')
            for policy in policyStr:
               token = re.split(',', policy)
               objs = jarray.array([String('security'),String(token[1])],Object)
               strs = jarray.array(['java.lang.String','java.lang.String'],String)
               # Remove the policy ref
               ora_mbs.invoke(portMbeanObj, 'removePolicyReference', objs,strs)
            # Remove all the call properties if token is blank
            if tokenType == '':
                helper.wcPrintMessage('REMOVE_WSSEC_PROPS')
                for x in portCallProps.items():
                    objs = jarray.array([String(x[0])],Object)
                    strs = jarray.array(['java.lang.String'],String)
                    ora_mbs.invoke(portMbeanObj, 'removeCallProperty', objs, strs)
                return 1
        except Exception, ex:
            helper.wcPrintErrorMessage('ERROR_REMOVING_WSSEC_PROPS')
            return 0

    # If token is passed add the policy reference
    if tokenType is not None:
        try:
            policy = policyName
            objs = jarray.array([String('security'),String(policy)],Object)
            strs = jarray.array(['java.lang.String','java.lang.String'],String)
            ora_mbs.invoke(portMbeanObj, 'addPolicyReference',objs,strs)
        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            helper.wcPrintErrorMessage('ERROR_UPDATING_POLICY_REF')
            return 0

    # If policy is not set for this port, no other ws-sec property will get updated
    policyStr = ora_mbs.getAttribute(portMbeanObj, 'PolicyReferencesAsStringArray')

    if len(policyStr) == 0:
       helper.wcPrintMessage('TOKEN_PROFILE_NOT_SET')
       return 0

    # Update the ws-sec properties and value on Port mbean
    for x in portCallProps.items():
        try:
            if x[1] == '':
                # Remove the call property if it exists
                objs = jarray.array([String(x[0])],Object)
                strs = jarray.array(['java.lang.String'],String)
                ora_mbs.invoke(portMbeanObj, 'removeCallProperty', objs, strs)

            elif x[1] is not None:
                # Add/update the call property
                objs = jarray.array([String(x[0]),String(x[1])],Object)
                strs = jarray.array(['java.lang.String','java.lang.String'],String)
                ora_mbs.invoke(portMbeanObj, 'addCallProperty', objs, strs)

        except Exception, ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            helper.wcPrintErrorMessage('ERROR_UPDATING_CALL_PROP_VALUE', x[1])
            return 0
    return 1

def listWSRPProducers(appName, name='', verbose=1, server='', applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    # String attributes to be listed on the WSRP connection
    attrs = [(helper.wcGetMessageFromResourceBundle('WSRP_CONNECTION_NAME'),'ConnectionName'),
             (helper.wcGetMessageFromResourceBundle('WSRP_WS_CONNECTION_NAME'),'WSConnectionName'),
             (helper.wcGetMessageFromResourceBundle('WSRP_PROXY_HOST'),'ProxyHost'),
             (helper.wcGetMessageFromResourceBundle('WSRP_PROXY_PORT'),'ProxyPort'),
             (helper.wcGetMessageFromResourceBundle('WSRP_TIMEOUT'),'Timeout')]

    # The associated WS connection to be listed
    wsConnectionMap = {'name':'WSConnectionName',
                       'type':'WebServiceConnection',
                       'attrs':[(helper.wcGetMessageFromResourceBundle('WSRP_WSDL_URL'),'WsdlUrl')]}

    if name != '':
        names=[name]
    else:
        names=[]
    producerhelper.listProducerAttrs(appName, attrs, 'ConnectionName', verbose, 'WSRPProducerConnection', wsConnectionMap, names)
        
    helper.webcenterRestore()

def deregisterWSRPProducer(appName, name,server='',applicationVersion=''):
    # Deregister the producer and delete its WSRP and WS connections. 
    helper.webcenterInitialize(server,applicationVersion)
    # Get the WSRP producer connection object with the given name
    wsrpObj = producerhelper.producerGetConnectionObject(appName, name, 'WSRPProducerConnection')
    if wsrpObj is None:
        # If the WSRP producer connection doesn't exist return without trying
        # to delete either connection
        print producerhelper.noProducerConnectionErrorMsg('WSRPProducerConnection', name)
        helper.webcenterRestore()
        return
    
    wsName = ora_mbs.getAttribute(wsrpObj, 'WSConnectionName')

    # Deregister the producer.    
    producerhelper.deregisterProducer(appName, name, 1, server, applicationVersion)

    # Delete the WSRP producer connection
    helper.deleteConnection(appName, name,server,applicationVersion)

    # Delete the WS connection
    helper.deleteConnection(appName, wsName,server,applicationVersion)

    helper.webcenterRestore()

def listWSRPProducerRegistrationProperties(appName, url, proxyHost=None, proxyPort=0,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    connectionName = 'temp-wsrp-' + UUID.randomUUID().toString()
    wsConnectionName = connectionName + '-wsconn'

    # Set up WSRP connection property dictionary
    wsrpConnProperties = {'WSConnectionName': wsConnectionName}
    if not proxyHost is None:
        wsrpConnProperties['ProxyHost'] =  proxyHost
        wsrpConnProperties['ProxyPort'] = proxyPort

    # Set up WS connection property dictionary
    wsConnProperties = {'WsdlUrl': url}

    model = _getWsdlModel(appName, url, proxyHost, proxyPort)
    if model is None:
        helper.webcenterRestore()
        return

    wsConnProperties['Model'] =  model

    # Create a java array consisting of CompositeData objects from each of the 
    # connection objects.  The mbean server will convert the CompositeData 
    # objects into ConnectionDescriptor objects.
    wsrpConnCompositeData = _createConnectionCompositeData(connectionName, 'WSRPProducerConnection', wsrpConnProperties)
    wsConnCompositeData = _createConnectionCompositeData(wsConnectionName, 'WebServiceConnection', wsConnProperties)
    connCompositeDataArray = jarray.array([wsrpConnCompositeData, wsConnCompositeData], Object)

    # Get the mbean name for the registration instance created with the 
    # connections.
    regInstName = producerhelper.createProducerRegistrationInstanceName(appName, connCompositeDataArray)

    if regInstName is None:
        helper.wcPrintErrorMessage('ERROR_PRODUCER_REG_INSTANCE')
        helper.webcenterRestore()
        return
    
    # Invoke the method getWSRPProducerRegistrationProperties method on the mbean
    # that has just been created.
    try:
        objs = jarray.array([String(connectionName)], Object)
        strs = jarray.array(['java.lang.String'], String)
        regPropertyArray = ora_mbs.invoke(producerhelper.getProducerObjectName(regInstName), 'getWSRPProducerRegistrationProperties', objs, strs)
        helper.wcPrintMessage('REG_PROPERTIES_TITLE')
        producerhelper.displayCompositeDataArray(regPropertyArray)
    except Exception, e:
        helper.wcPrintExceptionMessage(e.getMessage())
        helper.wcPrintErrorMessage('ERROR_GET_REG_PROPERTIES', regInstName)
        
    producerhelper.destroyProducerRegistrationInstance(appName, regInstName)
    helper.webcenterRestore()

def listWSRPProducerUserCategories(appName, name,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    userCatArray = _listUserCategories(appName, name)

    if userCatArray is None:
        # Error has occurred
        return

    producerhelper.displayCompositeDataArray(userCatArray)

    helper.webcenterRestore()

def _listUserCategories(appName, name):
    # Invoke listWSRPProducerUserCategories(connectionName) on the producer
    # manager.
    objs = jarray.array([String(name)], Object)
    strs = jarray.array(['java.lang.String'],String)

    userCatArray = None 
    try:
        userCatArray = ora_mbs.invoke(producerhelper.getProducerManager(appName),'listWSRPProducerUserCategories',objs,strs)
    except Exception, e:
        helper.wcPrintExceptionMessage(e.getMessage())
        helper.wcPrintErrorMessage('ERROR_LIST_USER_CATS', name)

    return userCatArray


def mapWSRPProducerUserCategory(appName, name, localRole, producerUserCategory,server='',applicationVersion=''):
    helper.webcenterInitialize(server,applicationVersion)
    if not _isValidProducerUserCategory(appName, name, producerUserCategory):
        helper.wcPrintMessage('USER_CAT_NAME_NOT_FOUND', producerUserCategory)
        return

    # Invoke mapWSRPProducerUserCategory(name, localRole, producerUserCategory)
    objs = jarray.array([String(name), String(localRole), String(producerUserCategory)], Object)
    strs = jarray.array(['java.lang.String', 'java.lang.String', 'java.lang.String'],String)
    try:
        ora_mbs.invoke(producerhelper.getProducerManager(appName), 'mapWSRPProducerUserCategory', objs, strs)
        helper.wcPrintMessage('MAPPED_ROLE_TO_USER_CAT', localRole + ',' + producerUserCategory) 
    except Exception, e:
        helper.wcPrintExceptionMessage(e.getMessage())
        helper.wcPrintErrorMessage('ERROR_MAPPING_ROLE_TO_USER_CAT', localRole + ',' + producerUserCategory)

    helper.webcenterRestore()

def _isValidProducerUserCategory(appName, name, producerUserCategory):

    # Check whether producerUserCategory exists
    userCatArray = _listUserCategories(appName, name)

    if userCatArray is None:
        return 0

    if len(userCatArray) > 0:
        for idx in range(len(userCatArray)):
            if producerUserCategory == userCatArray[idx].get("name"):
                return 1

    return 0


def _getWsdlModel(appName, url, proxyHost, proxyPort):
    # Must be called from within the webcenterInitialize and webcenterRestore
    # calls.

    # Call the method getWsdlModel on the producer manager mbean.  The method
    # arguments may or may not include proxy details.
    if proxyHost is None:
        objs = jarray.array([String(url)],Object)
        strs = jarray.array(['java.lang.String'],String)
    else:
        objs = jarray.array([String(url), String(proxyHost), int(proxyPort)], Object)
        strs = jarray.array(['java.lang.String', 'java.lang.String', 'int'], String)
         
    model = None
    try:
        model = ora_mbs.invoke(producerhelper.getProducerManager(appName),'getWsdlModel',objs,strs)
    except Exception, e:
        helper.wcPrintExceptionMessage(e.getMessage())
        helper.wcPrintErrorMessage('ERROR_RETRIEVING_WSDL', url)
    return model

def _createConnectionCompositeData(name, type, propmap):
    # Build the properties array from the supplied map with the key in column 0
    # and the value in column 1
    # String[][] properties = new String[propmap.size()][2];
    properties = Array.newInstance(String,[len(propmap), 2])
    i = 0
    for key,value in propmap.items():
        properties[i][0] = key
        properties[i][1] = str(value)
        i += 1

    compData = _createConnectionCompositeDataUsingArray(name, type, properties)
    return compData

def _createConnectionCompositeDataUsingArray(name, type, properties):
    itemNames = jarray.array([String('name'), String('type'), String('properties')], String)
    itemValues = jarray.array([name, type, properties], Object)
    return CompositeDataSupport(_getConnectionCompositeType(), itemNames, itemValues)

def _getConnectionCompositeType():
    itemNames = jarray.array([String('name'), String('type'), String('properties')], String)
    itemDescs = jarray.array([String('dummyDesc'), String('dummyDesc'), String('dummyDesc')], String)
    itemTypes = jarray.array([SimpleType.STRING, SimpleType.STRING, ArrayType(2, SimpleType.STRING)], OpenType)
    compType = CompositeType('oracle.portlet.client.mbean.ConnectionDescriptor', 'emptyDesc', itemNames, itemDescs, itemTypes) 
    return compType

def _createRegPropertyCompositeDataList(propDict):
    # Create a list of registration property CompositeData objects from the 
    # supplied dictionary of name-value pairs
    regPropList = [] 
    for key,value in propDict.items():
        cd = _createRegPropertyCompositeData(key, value)
        regPropList.append(cd)

    return regPropList

def _createRegPropertyCompositeData(name, value):
    # Create a WSRPRegistrationProperty with a name and value only
    itemNames = jarray.array([String('propertyName'), String('propertyLabel'), String('propertyHint'), String('propertyValue'), String('propertyLang')], String)
    itemValues = jarray.array([name, '', '', value, ''], Object)
    return CompositeDataSupport(_getRegPropertyCompositeType(), itemNames, itemValues)
 
def _getRegPropertyCompositeType():
    # A WSRPRegistrationProperty class has the fields name, label, hint, value,
    # and lang which are all strings.  The field names, e.g. propertyName
    # correspond to those used in the WSRPRegistrationProperty class.
    itemNames = jarray.array([String('propertyName'), String('propertyLabel'), String('propertyHint'), String('propertyValue'), String('propertyLang')], String)
    itemDescs = jarray.array([String('dummy'), String('dummy'), String('dummy'), String('dummy'), String('dummy')], String)
    itemTypes = jarray.array([SimpleType.STRING, SimpleType.STRING, SimpleType.STRING, SimpleType.STRING, SimpleType.STRING ], OpenType)
    compType = CompositeType('oracle.portlet.client.connection.wsrp.WSRPRegistrationProperty', 'emptyDesc', itemNames, itemDescs, itemTypes)
    return compType


def _setNoAuthOnNonMarkupWebServicePortMBeans(webServiceObjName):
    try:
        serviceONames = ora_mbs.getAttribute(webServiceObjName, 'ServiceMBeans');
    except Exception,ex:
        helper.wcPrintExceptionMessage(ex.getMessage())
        helper.wcPrintErrorMessage('ERROR_GETTING_SERVICE_MBEAN', message=webServiceObjName.toString())
        return None

    if serviceONames is None:
        return None
    for i in range(len(serviceONames)):
        try:
            portONames = ora_mbs.getAttribute(serviceONames[i], 'PortMBeans');
            for j in range(len(portONames)):
                portOName = portONames[j]
                if portOName is None:
                    continue
                bindName = ora_mbs.getAttribute(portOName, 'BindingName')
                name = bindName.getLocalPart()
                if not(name == 'WSRP_v2_Markup_Binding_SOAP' or name == 'WSRP_v1_Markup_Binding_SOAP'):
                    try:
                        # Add NoAuth Policy reference
                        objs = jarray.array([String('security'),String('oracle/no_authentication_client_policy')],Object)
                        strs = jarray.array(['java.lang.String','java.lang.String'],String)
                        ora_mbs.invoke(portOName, 'addPolicyReference',objs,strs)
                    except Exception, ex:
                        helper.wcPrintExceptionMessage(ex.getMessage())
                        helper.wcPrintErrorMessage('ERROR_ADDING_POLICY_REF')
                        return 0
        except Exception,ex:
            helper.wcPrintExceptionMessage(ex.getMessage())
            helper.wcPrintErrorMessage('ERROR_GETTING_PORT_FROM_WS_CONN', message=webServiceObjName.toString())
    return None


